﻿using System;
using System.Collections.Generic;
using System.Data.Services.Client;
using System.Linq;
using System.Windows;
using System.Windows.Browser;
using System.Windows.Controls;
using System.Windows.Media;
using SilverlightProjectTracker.RestClient;

namespace SilverlightProjectTracker
{
    public partial class EnterTimesheet : UserControl
    {
        readonly ProjectTrackerEntities client =
            new ProjectTrackerEntities(new Uri("WebServices/RestEntities.svc", UriKind.Relative));


        public EnterTimesheet()
        {
            InitializeComponent();
            LayoutRoot.ColumnDefinitions[1].MaxWidth = 0;
            BindingValidationError += EnterTimesheet_BindingValidationError;


            DataServiceQuery<Project> query = client.Projects;
            query.BeginExecute(ar => lbProjects.ItemsSource = query.EndExecute(ar), null);
        }

        #region Binding Validation

        readonly Dictionary<TextBox, bool> validationErrors = new Dictionary<TextBox, bool>();

        void EnterTimesheet_BindingValidationError(object sender, ValidationErrorEventArgs e)
        {
            if (!(e.Source is TextBox)) return;

            TextBox box = (TextBox) e.Source;

            bool isError = e.Action == ValidationErrorEventAction.Added;
            object tooltip = isError ? e.Error.Exception.Message : DependencyProperty.UnsetValue;
            ToolTipService.SetToolTip(box, tooltip);
            validationErrors[box] = isError;

            Color background = isError ? Colors.Red : Colors.White;
            box.Background = new SolidColorBrush(background);

            cmdSave.IsEnabled = validationErrors.Values.All(b => !b);
            e.Handled = true;
        }

        #endregion

        #region Selection Changed

        void lbProjects_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            IQueryable<Task> v = from p in client.Projects
                                 where p.ID == ((Project)lbProjects.SelectedItem).ID
                                 from t in p.Task
                                 select t;

            //have to run async:
            DataServiceQuery<Task> query = (DataServiceQuery<Task>) v;
            query.BeginExecute(ar => lbTasks.ItemsSource = query.EndExecute(ar), null);
        }

        void lbTasks_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            cmdSave.IsEnabled = lbTasks.SelectedItem != null;
            showBlankTimesheet();
        }

        void showBlankTimesheet()
        {
            Timesheet sheet = new Timesheet
                                  {
                                      Day = DateTime.Today,
                                      UserName = App.CurrentUserName,
                                      PercentComplete = 50,
                                  };
            //unhide the column:
            LayoutRoot.ColumnDefinitions[1].MaxWidth = double.MaxValue;
            gridTimesheet.DataContext = sheet;
        }

        #endregion

        #region Save

        void cmdSave_Click(object sender, RoutedEventArgs e)
        {
            Timesheet sheet = (Timesheet) gridTimesheet.DataContext;
            client.AddToTimesheets(sheet);
            sheet.Task = (Task) lbTasks.SelectedItem;
            //have to explicitly create this link:
            client.AddLink(sheet, "Task", lbTasks.SelectedItem);
            cmdSave.IsEnabled = false;
            client.BeginSaveChanges(SaveChangesOptions.Batch, SaveChangesCallBack, null);
        }


        void SaveChangesCallBack(IAsyncResult ar)
        {
            DataServiceResponse response = client.EndSaveChanges(ar);
            IEnumerable<string> errors = from or in response
                                         from ed in or.OfType<EntityDescriptor>()
                                         where ed.Error != null
                                         select ed.Error.ToString();

            if (errors.Count() > 0)
            {
                HtmlPage.Window.Invoke("alert", string.Join(Environment.NewLine, errors.ToArray()));
            }
            else
            {
                HtmlPage.Window.Invoke("alert", "Your timesheet was succesfully submitted");
            }
            showBlankTimesheet();
            cmdSave.IsEnabled = true;
        }

        #endregion
    }
}