﻿using System;
using System.Data.Services.Client;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using SilverlightProjectTracker.RestClient;

namespace SilverlightProjectTracker
{
    public partial class Overview : UserControl
    {
        readonly ProjectTrackerEntities client =
            new ProjectTrackerEntities(new Uri("WebServices/RestEntities.svc", UriKind.Relative))
                {MergeOption = MergeOption.OverwriteChanges};

        public Overview()
        {
            InitializeComponent();
            //read the projects
            DataServiceQuery<Project> query = client.Projects;
            query.BeginExecute(ar => lbProjects.ItemsSource = query.EndExecute(ar), null);
        }

        void lbProjects_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            //read the tasks for the selected project
            IQueryable<Task> v = from p in client.Projects
                                 where p.ID == ((Project) lbProjects.SelectedItem).ID
                                 from t in p.Task
                                 select t;

            //have to run async:
            DataServiceQuery<Task> query = (DataServiceQuery<Task>) v;
            query = query.Expand("Timesheet");
            query.BeginExecute(ar => lbTasks.ItemsSource = query.EndExecute(ar), null);
        }

        void gTask_Loaded(object sender, RoutedEventArgs e)
        {
            Grid source = (Grid) sender;
            Task task = (Task) source.DataContext;
            Project project = (Project) lbProjects.SelectedItem;
            double preTaskDays = (task.ExpectedStart - project.ExpectedStart).TotalDays;
            double taskDays = (task.ExpectedFinish - task.ExpectedStart).TotalDays;
            double postTaskDays = (project.ExpectedFinish - task.ExpectedFinish).TotalDays;

            source.ColumnDefinitions[0].Width = new GridLength(preTaskDays, GridUnitType.Star);
            source.ColumnDefinitions[1].Width = new GridLength(taskDays, GridUnitType.Star);
            source.ColumnDefinitions[2].Width = new GridLength(postTaskDays, GridUnitType.Star);
        }

        void gTimeSheet_Loaded(object sender, RoutedEventArgs e)
        {
            Grid source = (Grid) sender;
            Timesheet timesheet = (Timesheet) source.DataContext;
            Project project = (Project) lbProjects.SelectedItem;
            double preTimesheetDays = (timesheet.Day - project.ExpectedStart).TotalDays;
            double timesheetDays = 1;
            double postTimesheetDays = (project.ExpectedFinish - timesheet.Day).TotalDays - 1;

            source.ColumnDefinitions[0].Width = new GridLength(preTimesheetDays, GridUnitType.Star);
            source.ColumnDefinitions[1].Width = new GridLength(timesheetDays, GridUnitType.Star);
            source.ColumnDefinitions[2].Width = new GridLength(postTimesheetDays, GridUnitType.Star);
        }

        void rTimesheet_MouseEnter(object sender, MouseEventArgs e)
        {
            FrameworkElement el = (FrameworkElement) sender;
            Timesheet timesheet = (Timesheet) el.DataContext;
            Grid g;
            do
            {
                g = el.FindName("gTaskComplete") as Grid;
                el = (FrameworkElement) VisualTreeHelper.GetParent(el);
            } while (g == null);
            g.ColumnDefinitions[0].Width = new GridLength(timesheet.PercentComplete, GridUnitType.Star);
            g.ColumnDefinitions[1].Width = new GridLength(100 - timesheet.PercentComplete, GridUnitType.Star);
        }

        void rTimesheet_MouseLeave(object sender, MouseEventArgs e)
        {
            FrameworkElement el = (FrameworkElement) sender;
            Grid g;
            do
            {
                g = el.FindName("gTaskComplete") as Grid;
                el = (FrameworkElement) VisualTreeHelper.GetParent(el);
            } while (g == null);
            g.ColumnDefinitions[0].Width = new GridLength(0);
        }
    }
}